/*
** Module   : Commitable.java
** Abstract : the interface for objects whose changes can be externally 
**            committed
**
** Copyright (c) 2005-2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------------------Description-----------------------------
** 001 GES 20050811   @22108 Created initial version, providing an 
**                           interface for objects whose changes must be
**                           externally committed (saved) on successful
**                           completion of a block.
** 002 GES 20051007   @23006 Added a rollback() notification method
**                           which occurs after all undo processing is
**                           complete and which allows custom internal
**                           state to be maintained.
** 003 CA  20080826   @39566 Added rollbackPending() notification method
**                           which notifies the nested blocks when the 
**                           target for an UNDO statement is one of its
**                           parent block.
** 004 GES 20090729   @43441 Added validate() notification which occurs just
**                           before any normal exit from a block.
** 005 OM  20140213          Added extra parameter to validate to identify external procedures.
*/

package com.goldencode.p2j.util;

/**
 * An interface for objects whose changes can be externally committed (saved)
 * on successful completion of a block or rolled back (undone) on failure.
 */
public interface Commitable
{
   /**
    * Saves any temporary state to the persistent form or storage that is
    * associated with this class.
    *
    * @param    transaction
    *           <code>true</code> if this is a full transaction and
    *           <code>false</code> if this is only a sub-transaction (a
    *           nested scope with transaction support).
    */
   public void commit(boolean transaction);
   
   /**
    * Notifies that a rollback has just occurred (all state maintained
    * through the {@link Undoable} interface has been already rolled back).
    * This notification can be used to implement cleanup of any custom
    * state that needs to be maintained in this case.  It can also be used
    * to implement a rollback that is completely independent of the
    * <code>Undoable</code> (externally driven) interface.
    *
    * @param    transaction
    *           <code>true</code> if this is a full transaction and
    *           <code>false</code> if this is only a sub-transaction (a
    *           nested scope with transaction support).
    */
   public void rollback(boolean transaction);

   /**
    * Notifies that a pending rollback has just occurred (i.e. a UNDO targets
    * a parent block). This can be used to rollup any information up the
    * parent stack, so the targeted block will be able to process.
    */
   public void rollbackPending();
   
   /**
    * Notifies that a block is about to be normally exited or normally
    * iterated and that any deferred validation processing should be executed.
    * A normal exit or iteration is one that occurs naturally by hitting the
    * end of the block (or loop). A <code>LEAVE</code>, <code>NEXT</code> and 
    * <code>RETURN</code> are also considered normal exits or iterations so
    * long as they were not preceded by an <code>UNDO</code>. Likewise, a
    * <code>QUIT</code> is a normal exit so long as it is not inside a block
    * that has an <code>ON QUIT UNDO, &lt;action&gt;</code>.
    *
    * @param   transaction
    *          <code>true</code> if this is a full transaction and
    *          <code>false</code> if this is only a sub-transaction (a
    *          nested scope with transaction support).
    * @param   externalBlock
    *          Identifies external procedure blocks.
    *
    * @throws   ErrorConditionException
    *           If validation fails.
    */
   public void validate(boolean transaction, boolean externalBlock)
   throws ErrorConditionException;
}
